#ifndef __VARIANCECOMPONENTS_H__
#define __VARIANCECOMPONENTS_H__

#include "MathNormal.h"
#include "Pedigree.h"
#include "QtlModel.h"

class MerlinPDF;
class FamilyAnalysis;
class NormalEquations;

class VarianceComponents
   {
   public:
      VarianceComponents();

      void Analyse(FamilyAnalysis & engine);

      static bool   useCovariates;
      static bool   useProbands;
      static double unlinkedFraction;

      static QtlModel customModels;

      void OpenPerFamilyFile();
      void ClosePerFamilyFile();

   protected:
      void ListCovariates();
      void PrintCovariates();

      void PrintPvalue(double pvalue);

      bool CheckCovariates(Person & p);
      bool CheckRepeatedMeasures(Pedigree & ped, IntArray & traitIndex, IntArray * pheno);
      bool CheckAveragedMeasures(Pedigree & ped, IntArray * pheno, int repeatedMeasures);

      void SetupPDF(MerlinPDF & pdf, String & trait, int probandCount);

      void DuplicateModel(NormalEquations & dest, NormalEquations & src,
                          int vc_count, int beta_count);
      double TotalVariance(Vector & variances, int vc_count);

      FILE * perFamily;

      void WritePerFamilyLOD(Pedigree & ped, IntArray * pheno,
           const char * position, Vector & nullLLK, Vector & fullLLK);
      void WritePerFamilyLOD(Pedigree & ped, IntArray * pheno,
           const char * position, const char * label, Vector & nullLLK, Vector & fullLLK);

      IntArray covariates;

      // Utility functions for dealing with repeated measurements,
      // when they have not been averaged
      bool   SkipTrait(int trait);
      bool   IsPhenotyped(Person & person, IntArray & measurements, int trait);
      bool   IndexMeasurements(IntArray & index, String & label, int trait);
      double RetrieveMeasurement(Person & person, IntArray & measurements, int & measurement);
      bool   ExtractMeasurementNumber(const String & label, int & prefix_len, int & number);
      int    FindMeasurement(const String & label, int measurement);
      void   SummarizeRepeatedMeasures(Pedigree & ped, String & label, IntArray * pheno);
   };

#endif

